@@ -92,7 +92,7 @@ where
     /// Handle an object path being added to the tree of D-Bus objects.
     fn handle_add_action(
         &self,
-        mut write_lock: TreeWriteLock<'_, E>,
+        mut write_lock: TreeWriteLock<E>,
         path: ObjectPath<MTSync<TData<E>>, TData<E>>,
         interfaces: InterfacesAddedThreadSafe,
     ) {
@@ -106,7 +106,7 @@ where
     /// Handle an object path being removed from the tree of D-Bus objects.
     fn handle_remove_action(
         &self,
-        mut write_lock: TreeWriteLock<'_, E>,
+        mut write_lock: TreeWriteLock<E>,
         path: Path<'static>,
         interfaces: InterfacesRemoved,
     ) {
@@ -167,7 +167,7 @@ where
     /// Handle a pool name change in the engine.
     fn handle_pool_name_change(
         &self,
-        read_lock: TreeReadLock<'_, E>,
+        read_lock: TreeReadLock<E>,
         item: Path<'static>,
         new_name: String,
     ) {
@@ -324,7 +324,7 @@ where
     #[allow(clippy::option_option)]
     fn handle_fs_background_change(
         &self,
-        read_lock: TreeReadLock<'_, E>,
+        read_lock: TreeReadLock<E>,
         uuid: FilesystemUuid,
         new_size: Option<Bytes>,
         new_used: Option<Option<Bytes>>,
@@ -379,7 +379,7 @@ where
     #[allow(clippy::option_option)]
     fn handle_pool_background_change(
         &self,
-        read_lock: TreeReadLock<'_, E>,
+        read_lock: TreeReadLock<E>,
         uuid: PoolUuid,
         new_used: Option<Option<Bytes>>,
         new_alloc: Option<Bytes>,
@@ -448,6 +448,44 @@ where
         }
     }
 
+    /// Send a signal indicating that the pool total allocated size has changed.
+    fn handle_pool_alloc_size_change(&self, path: Path<'static>, new_alloc_size: Bytes) {
+        if let Err(e) = self.property_changed_invalidated_signal(
+            &path,
+            once((
+                consts::POOL_ALLOC_SIZE_PROP.to_string(),
+                box_variant!(pool_alloc_to_prop(new_alloc_size)),
+            ))
+            .collect::<HashMap<_, _>>(),
+            vec![],
+            &consts::standard_pool_interfaces(),
+        ) {
+            warn!(
+                "Failed to send a signal over D-Bus indicating pool allocated size change: {}",
+                e
+            );
+        }
+    }
+
+    /// Send a signal indicating that the pool usage has changed.
+    fn handle_pool_usage_change(&self, path: Path<'static>, new_usage: Option<Bytes>) {
+        if let Err(e) = self.property_changed_invalidated_signal(
+            &path,
+            once((
+                consts::POOL_TOTAL_USED_PROP.to_string(),
+                box_variant!(pool_used_to_prop(new_usage)),
+            ))
+            .collect::<HashMap<_, _>>(),
+            vec![],
+            &consts::standard_pool_interfaces(),
+        ) {
+            warn!(
+                "Failed to send a signal over D-Bus indicating pool allocated size change: {}",
+                e
+            );
+        }
+    }
+
     /// Handle a D-Bus action that has been generated by the connection processing
     /// handle.
     fn handle_dbus_action(&mut self, action: DbusAction<E>) -> StratisResult<bool> {
@@ -530,6 +568,14 @@ where
                 self.handle_locked_pools_change(pools);
                 Ok(true)
             }
+            DbusAction::PoolAllocSizeChange(item, new_alloc) => {
+                self.handle_pool_alloc_size_change(item, new_alloc);
+                Ok(true)
+            }
+            DbusAction::PoolUsedChange(item, new_usage) => {
+                self.handle_pool_usage_change(item, new_usage);
+                Ok(true)
+            }
         }
     }
 
