@@ -26,7 +26,7 @@ use crate::{
         types::{
             ActionAvailability, BlockDevTier, Clevis, CreateAction, DeleteAction, DevUuid,
             EncryptionInfo, FilesystemUuid, Key, KeyDescription, Name, PoolEncryptionInfo,
-            PoolUuid, Redundancy, RegenAction, RenameAction, SetCreateAction, SetDeleteAction,
+            PoolUuid, RegenAction, RenameAction, SetCreateAction, SetDeleteAction, StratPoolDiff,
         },
     },
     stratis::{StratisError, StratisResult},
@@ -37,16 +37,10 @@ pub struct SimPool {
     block_devs: HashMap<DevUuid, SimDev>,
     cache_devs: HashMap<DevUuid, SimDev>,
     filesystems: Table<FilesystemUuid, SimFilesystem>,
-    #[allow(dead_code)]
-    redundancy: Redundancy,
 }
 
 impl SimPool {
-    pub fn new(
-        paths: &[&Path],
-        redundancy: Redundancy,
-        enc_info: Option<&EncryptionInfo>,
-    ) -> (PoolUuid, SimPool) {
+    pub fn new(paths: &[&Path], enc_info: Option<&EncryptionInfo>) -> (PoolUuid, SimPool) {
         let devices: HashSet<_, RandomState> = HashSet::from_iter(paths);
         let device_pairs = devices.iter().map(|p| SimDev::new(p, enc_info));
         (
@@ -55,7 +49,6 @@ impl SimPool {
                 block_devs: device_pairs.collect(),
                 cache_devs: HashMap::new(),
                 filesystems: Table::default(),
-                redundancy,
             },
         )
     }
@@ -258,7 +251,7 @@ impl Pool for SimPool {
         _pool_name: &str,
         paths: &[&Path],
         tier: BlockDevTier,
-    ) -> StratisResult<SetCreateAction<DevUuid>> {
+    ) -> StratisResult<(SetCreateAction<DevUuid>, Option<StratPoolDiff>)> {
         validate_paths(paths)?;
 
         if tier == BlockDevTier::Cache && !self.has_cache() {
@@ -269,7 +262,7 @@ impl Pool for SimPool {
 
         if paths.is_empty() {
             // Treat adding no new blockdev as the empty set.
-            return Ok(SetCreateAction::new(vec![]));
+            return Ok((SetCreateAction::new(vec![]), None));
         }
 
         let devices: HashSet<_, RandomState> = HashSet::from_iter(paths);
@@ -307,7 +300,7 @@ impl Pool for SimPool {
         };
 
         the_vec.extend(filtered_device_pairs);
-        Ok(SetCreateAction::new(ret_uuids))
+        Ok((SetCreateAction::new(ret_uuids), None))
     }
 
     fn bind_clevis(
@@ -560,8 +553,8 @@ impl Pool for SimPool {
         Sectors(5 * IEC::Mi)
     }
 
-    fn total_physical_used(&self) -> StratisResult<Sectors> {
-        Ok(Sectors(0))
+    fn total_physical_used(&self) -> Option<Sectors> {
+        Some(Sectors(0))
     }
 
     fn filesystems(&self) -> Vec<(Name, FilesystemUuid, &Self::Filesystem)> {
@@ -647,30 +640,28 @@ mod tests {
 
     use std::path::Path;
 
-    use crate::engine::Engine;
-
-    use crate::engine::sim_engine::SimEngine;
-
-    use crate::engine::types::EngineAction;
+    use crate::engine::{
+        sim_engine::SimEngine,
+        types::{EngineAction, LockKey},
+        Engine,
+    };
 
     use super::*;
 
     #[test]
     /// Renaming a filesystem on an empty pool always works
     fn rename_empty() {
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         assert_matches!(
             pool.rename_filesystem(pool_name, FilesystemUuid::new_v4(), "new_name"),
             Ok(RenameAction::NoSource)
@@ -680,19 +671,17 @@ mod tests {
     #[test]
     /// Renaming a filesystem to another filesystem should work if new name not taken
     fn rename_happens() {
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         let infos = pool
             .create_filesystems(pool_name, uuid, &[("old_name", None)])
             .unwrap()
@@ -710,19 +699,17 @@ mod tests {
     fn rename_fails() {
         let old_name = "old_name";
         let new_name = "new_name";
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         let results = pool
             .create_filesystems(pool_name, uuid, &[(old_name, None), (new_name, None)])
             .unwrap()
@@ -739,19 +726,17 @@ mod tests {
     /// Renaming should succeed if old_name absent, new present
     fn rename_no_op() {
         let new_name = "new_name";
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         assert_matches!(
             pool.rename_filesystem(pool_name, FilesystemUuid::new_v4(), new_name),
             Ok(RenameAction::NoSource)
@@ -761,19 +746,17 @@ mod tests {
     #[test]
     /// Removing an empty list of filesystems should always succeed
     fn destroy_fs_empty() {
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         assert!(match pool.destroy_filesystems(pool_name, &[]) {
             Ok(uuids) => !uuids.is_changed(),
             _ => false,
@@ -783,19 +766,17 @@ mod tests {
     #[test]
     /// Removing a non-empty list of filesystems should succeed on empty pool
     fn destroy_fs_some() {
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         assert_matches!(
             pool.destroy_filesystems(pool_name, &[FilesystemUuid::new_v4()]),
             Ok(_)
@@ -805,19 +786,17 @@ mod tests {
     #[test]
     /// Removing a non-empty list of filesystems should succeed on any pool
     fn destroy_fs_any() {
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         let fs_results = pool
             .create_filesystems(pool_name, uuid, &[("fs_name", None)])
             .unwrap()
@@ -833,19 +812,17 @@ mod tests {
     #[test]
     /// Creating an empty list of filesystems should succeed, always
     fn create_fs_none() {
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         let fs = pool.create_filesystems(pool_name, uuid, &[]).unwrap();
         assert!(!fs.is_changed())
     }
@@ -853,19 +830,17 @@ mod tests {
     #[test]
     /// Creating a non-empty list of filesystems always succeeds.
     fn create_fs_some() {
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         assert!(match pool
             .create_filesystems(pool_name, uuid, &[("name", None)])
             .ok()
@@ -880,19 +855,17 @@ mod tests {
     /// Creating a an already existing filesystem fails.
     fn create_fs_conflict() {
         let fs_name = "fs_name";
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         pool.create_filesystems(pool_name, uuid, &[(fs_name, None)])
             .unwrap();
         let set_create_action = pool
@@ -905,19 +878,17 @@ mod tests {
     /// Requesting identical filesystems succeeds.
     fn create_fs_dups() {
         let fs_name = "fs_name";
-        let mut engine = SimEngine::default();
+        let engine = SimEngine::default();
         let pool_name = "pool_name";
-        let uuid = engine
-            .create_pool(
-                pool_name,
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let pool = engine.get_mut_pool(uuid).unwrap().1;
+        let uuid = test_async!(engine.create_pool(
+            pool_name,
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let mut pool = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
         assert!(match pool
             .create_filesystems(pool_name, uuid, &[(fs_name, None), (fs_name, None)])
             .ok()
@@ -931,23 +902,22 @@ mod tests {
     #[test]
     /// Adding a list of devices to an empty pool should yield list.
     fn add_device_empty() {
-        let mut engine = SimEngine::default();
-        let uuid = engine
-            .create_pool(
-                "pool_name",
-                strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
-                None,
-                None,
-            )
-            .unwrap()
-            .changed()
-            .unwrap();
-        let (pool_name, pool) = engine.get_mut_pool(uuid).unwrap();
+        let engine = SimEngine::default();
+        let uuid = test_async!(engine.create_pool(
+            "pool_name",
+            strs_to_paths!(["/dev/one", "/dev/two", "/dev/three"]),
+            None,
+        ))
+        .unwrap()
+        .changed()
+        .unwrap();
+        let guard = test_async!(engine.get_mut_pool(LockKey::Uuid(uuid))).unwrap();
+        let (pool_name, _, pool) = guard.as_tuple();
         let devices = [Path::new("/s/a"), Path::new("/s/b")];
         assert!(match pool
             .add_blockdevs(uuid, &*pool_name, &devices, BlockDevTier::Data)
             .ok()
-            .and_then(|c| c.changed())
+            .and_then(|c| c.0.changed())
         {
             Some(devs) => devs.len() == devices.len(),
             _ => false,
