@@ -20,7 +20,5 @@ pub use self::{
         crypt_metadata_size, CryptActivationHandle, CryptHandle, CryptMetadataHandle,
         CLEVIS_TANG_TRUST_URL,
     },
+    devices::{initialize_devices, process_and_verify_devices},
 };
-
-#[cfg(test)]
-pub use self::devices::{initialize_devices, process_and_verify_devices};
