@@ -12,7 +12,7 @@ use stratisd_proc_macros::strat_pool_impl_gen;
 
 use crate::{
     engine::{
-        engine::Pool,
+        engine::{DumpState, Pool, StateDiff},
         shared::{
             init_cache_idempotent_or_err, validate_filesystem_size_specs, validate_name,
             validate_paths,
@@ -21,13 +21,15 @@ use crate::{
             backstore::{Backstore, StratBlockDev},
             metadata::MDADataSize,
             serde_structs::{FlexDevsSave, PoolSave, Recordable},
-            thinpool::{StratFilesystem, ThinPool, ThinPoolSizeParams, DATA_BLOCK_SIZE},
+            thinpool::{
+                StratFilesystem, ThinPool, ThinPoolSizeParams, ThinPoolState, DATA_BLOCK_SIZE,
+            },
         },
         types::{
             ActionAvailability, BlockDevTier, Clevis, CreateAction, DeleteAction, DevUuid,
             EncryptionInfo, FilesystemUuid, Key, KeyDescription, Name, PoolEncryptionInfo,
-            PoolUuid, Redundancy, RegenAction, RenameAction, SetCreateAction, SetDeleteAction,
-            StratFilesystemDiff, ThinPoolDiff,
+            PoolUuid, RegenAction, RenameAction, SetCreateAction, SetDeleteAction,
+            StratFilesystemDiff, StratPoolDiff,
         },
     },
     stratis::{StratisError, StratisResult},
@@ -136,10 +138,9 @@ fn check_metadata(metadata: &PoolSave) -> StratisResult<()> {
 #[derive(Debug)]
 pub struct StratPool {
     backstore: Backstore,
-    #[allow(dead_code)]
-    redundancy: Redundancy,
     thin_pool: ThinPool,
     action_avail: ActionAvailability,
+    usage: Option<Sectors>,
 }
 
 #[strat_pool_impl_gen]
@@ -151,7 +152,6 @@ impl StratPool {
     pub fn initialize(
         name: &str,
         paths: &[&Path],
-        redundancy: Redundancy,
         encryption_info: Option<&EncryptionInfo>,
     ) -> StratisResult<(PoolUuid, StratPool)> {
         let pool_uuid = PoolUuid::new_v4();
@@ -169,7 +169,7 @@ impl StratPool {
             &mut backstore,
         );
 
-        let mut thinpool = match thinpool {
+        let thinpool = match thinpool {
             Ok(thinpool) => thinpool,
             Err(err) => {
                 let _ = backstore.destroy();
@@ -177,13 +177,14 @@ impl StratPool {
             }
         };
 
-        thinpool.check(pool_uuid, &mut backstore)?;
-
+        let used = thinpool
+            .used()
+            .map(|u| u + backstore.datatier_metadata_size());
         let mut pool = StratPool {
             backstore,
-            redundancy,
             thin_pool: thinpool,
             action_avail: ActionAvailability::Full,
+            usage: used,
         };
 
         pool.write_metadata(&Name::new(name.to_owned()))?;
@@ -224,13 +225,16 @@ impl StratPool {
             &backstore,
         )?;
 
-        let (needs_save, _) = thinpool.check(uuid, &mut backstore)?;
+        let needs_save = thinpool.check(uuid, &mut backstore)?;
 
+        let used = thinpool
+            .used()
+            .map(|u| u + backstore.datatier_metadata_size());
         let mut pool = StratPool {
             backstore,
-            redundancy: Redundancy::NONE,
             thin_pool: thinpool,
             action_avail,
+            usage: used,
         };
 
         if needs_save {
@@ -279,8 +283,10 @@ impl StratPool {
         &mut self,
         pool_uuid: PoolUuid,
         pool_name: &Name,
-    ) -> StratisResult<ThinPoolDiff> {
-        let (changed, diff) = self.thin_pool.check(pool_uuid, &mut self.backstore)?;
+    ) -> StratisResult<StratPoolDiff> {
+        let cached = self.cached();
+        let changed = self.thin_pool.check(pool_uuid, &mut self.backstore)?;
+        let diff = cached.diff(&self.dump(()));
         if changed {
             self.write_metadata(pool_name)?;
         }
@@ -297,8 +303,6 @@ impl StratPool {
     ) -> StratisResult<HashMap<FilesystemUuid, StratFilesystemDiff>> {
         let changed = self.thin_pool.check_fs(pool_uuid)?;
         if !changed.is_empty() {
-            // Do we really need this if we call event_on after doing the
-            // filesystem checks?
             self.write_metadata(pool_name)?;
         }
         Ok(changed)
@@ -393,13 +397,6 @@ impl<'a> Into<Value> for &'a StratPool {
     }
 }
 
-pub fn total_physical_used(
-    backstore: &Backstore,
-    size: StratisResult<Sectors>,
-) -> StratisResult<Sectors> {
-    size.map(|v| v + backstore.datatier_metadata_size())
-}
-
 #[strat_pool_impl_gen]
 impl Pool for StratPool {
     type Filesystem = StratFilesystem;
@@ -573,7 +570,7 @@ impl Pool for StratPool {
         pool_name: &str,
         paths: &[&Path],
         tier: BlockDevTier,
-    ) -> StratisResult<SetCreateAction<DevUuid>> {
+    ) -> StratisResult<(SetCreateAction<DevUuid>, Option<StratPoolDiff>)> {
         validate_paths(paths)?;
 
         let bdev_info = if tier == BlockDevTier::Cache && !self.has_cache() {
@@ -586,7 +583,7 @@ impl Pool for StratPool {
             ));
         } else if paths.is_empty() {
             //TODO: Substitute is_empty check with process_and_verify_devices
-            return Ok(SetCreateAction::new(vec![]));
+            return Ok((SetCreateAction::new(vec![]), None));
         } else if tier == BlockDevTier::Cache {
             // If adding cache devices, must suspend the pool; the cache
             // must be augmented with the new devices.
@@ -594,7 +591,7 @@ impl Pool for StratPool {
             let bdev_info_res = self.backstore.add_cachedevs(pool_uuid, paths);
             self.thin_pool.resume()?;
             let bdev_info = bdev_info_res?;
-            Ok(SetCreateAction::new(bdev_info))
+            Ok((SetCreateAction::new(bdev_info), None))
         } else {
             // If just adding data devices, no need to suspend the pool.
             // No action will be taken on the DM devices.
@@ -606,8 +603,18 @@ impl Pool for StratPool {
             // addition of the new data devs may have changed its context
             // so that it can satisfy the allocation request where
             // previously it could not. Run check() in case that is true.
-            self.thin_pool.check(pool_uuid, &mut self.backstore)?;
-            Ok(SetCreateAction::new(bdev_info))
+            let cached = self.cached();
+            let check = match self.thin_pool.check(pool_uuid, &mut self.backstore) {
+                Ok(_) => {
+                    let diff = cached.diff(&self.dump(()));
+                    Some(diff)
+                }
+                Err(e) => {
+                    warn!("Failed to check the thin pool for status changes; some information may not be able to reported to the IPC layer: {}", e);
+                    None
+                }
+            };
+            Ok((SetCreateAction::new(bdev_info), check))
         };
         self.write_metadata(pool_name)?;
         bdev_info
@@ -678,14 +685,14 @@ impl Pool for StratPool {
         self.backstore.datatier_allocated_size()
     }
 
-    fn total_physical_used(&self) -> StratisResult<Sectors> {
+    fn total_physical_used(&self) -> Option<Sectors> {
         // TODO: note that with the addition of another layer, the
         // calculation of the amount of physical spaced used by means
         // of adding the amount used by Stratis metadata to the amount used
         // by the pool abstraction will be invalid. In the event of, e.g.,
         // software RAID, the amount will be far too low to be useful, in the
         // event of, e.g, VDO, the amount will be far too large to be useful.
-        total_physical_used(&self.backstore, self.thin_pool.total_physical_used())
+        self.usage
     }
 
     fn filesystems(&self) -> Vec<(Name, FilesystemUuid, &Self::Filesystem)> {
@@ -758,6 +765,51 @@ impl Pool for StratPool {
     }
 }
 
+pub struct StratPoolState {
+    usage: Option<Bytes>,
+    thin_pool: ThinPoolState,
+}
+
+impl StateDiff for StratPoolState {
+    type Diff = StratPoolDiff;
+
+    fn diff(&self, other: &Self) -> Self::Diff {
+        StratPoolDiff {
+            usage: if self.usage != other.usage {
+                Some(other.usage)
+            } else {
+                None
+            },
+            thin_pool: self.thin_pool.diff(&other.thin_pool),
+        }
+    }
+}
+
+impl<'a> DumpState<'a> for StratPool {
+    type State = StratPoolState;
+    type DumpInput = ();
+
+    fn cached(&self) -> Self::State {
+        StratPoolState {
+            thin_pool: self.thin_pool.cached(),
+            usage: self.usage.map(|u| u.bytes()),
+        }
+    }
+
+    fn dump(&mut self, _: ()) -> Self::State {
+        let thin_pool = self.thin_pool.dump(&self.backstore);
+        let usage = self
+            .thin_pool
+            .used()
+            .map(|u| u + self.backstore.datatier_metadata_size());
+        self.usage = usage;
+        StratPoolState {
+            usage: usage.map(|u| u.bytes()),
+            thin_pool,
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use std::{
@@ -767,15 +819,16 @@ mod tests {
 
     use nix::mount::{mount, umount, MsFlags};
 
-    use devicemapper::{Bytes, ThinPoolStatus, ThinPoolStatusSummary, IEC, SECTOR_SIZE};
+    use devicemapper::{Bytes, IEC, SECTOR_SIZE};
 
     use crate::engine::{
         engine::Filesystem,
         strat_engine::{
             cmd::udev_settle,
             tests::{loopbacked, real},
+            thinpool::ThinPoolStatusDigest,
         },
-        types::{EngineAction, Redundancy},
+        types::EngineAction,
     };
 
     use super::*;
@@ -809,7 +862,7 @@ mod tests {
     fn test_empty_pool(paths: &[&Path]) {
         assert_eq!(paths.len(), 0);
         assert_matches!(
-            StratPool::initialize("stratis_test_pool", paths, Redundancy::NONE, None),
+            StratPool::initialize("stratis_test_pool", paths, None),
             Err(_)
         );
     }
@@ -833,7 +886,7 @@ mod tests {
         let (paths1, paths2) = paths.split_at(paths.len() / 2);
 
         let name = "stratis-test-pool";
-        let (uuid, mut pool) = StratPool::initialize(name, paths2, Redundancy::NONE, None).unwrap();
+        let (uuid, mut pool) = StratPool::initialize(name, paths2, None).unwrap();
         invariant(&pool, name);
 
         let metadata1 = pool.record(name);
@@ -916,8 +969,7 @@ mod tests {
         let (data_path, data_paths) = data_paths.split_at(1);
 
         let name = "stratis-test-pool";
-        let (uuid, mut pool) =
-            StratPool::initialize(name, data_path, Redundancy::NONE, None).unwrap();
+        let (uuid, mut pool) = StratPool::initialize(name, data_path, None).unwrap();
         invariant(&pool, name);
 
         pool.init_cache(uuid, name, cache_path).unwrap();
@@ -953,8 +1005,7 @@ mod tests {
         let (paths1, paths2) = paths.split_at(1);
 
         let name = "stratis-test-pool";
-        let (pool_uuid, mut pool) =
-            StratPool::initialize(name, paths1, Redundancy::NONE, None).unwrap();
+        let (pool_uuid, mut pool) = StratPool::initialize(name, paths1, None).unwrap();
         invariant(&pool, name);
 
         let fs_name = "stratis_test_filesystem";
@@ -978,20 +1029,14 @@ mod tests {
 
             let mut amount_written = Sectors(0);
             let buffer_length = Bytes::from(buffer_length).sectors();
-            while match pool.thin_pool.state() {
-                Some(ThinPoolStatus::Working(working)) => {
-                    working.summary == ThinPoolStatusSummary::Good
-                }
-                _ => false,
-            } {
+            while matches!(pool.thin_pool.state(), Some(ThinPoolStatusDigest::Good)) {
                 f.write_all(buf).unwrap();
                 amount_written += Sectors(1);
                 // Run check roughly every time the buffer is cleared.
                 // Running it more often is pointless as the pool is guaranteed
                 // not to see any effects unless the buffer is cleared.
                 if amount_written % buffer_length == Sectors(1) {
-                    pool.thin_pool
-                        .check(pool_uuid, &mut pool.backstore)
+                    pool.event_on(pool_uuid, &Name::new(name.to_string()))
                         .unwrap();
                 }
             }
@@ -1000,9 +1045,7 @@ mod tests {
                 .unwrap();
 
             match pool.thin_pool.state() {
-                Some(ThinPoolStatus::Working(working)) => {
-                    assert_eq!(working.summary, ThinPoolStatusSummary::Good)
-                }
+                Some(ThinPoolStatusDigest::Good) => (),
                 _ => panic!("thin pool status should be back to working"),
             }
         }
@@ -1034,7 +1077,7 @@ mod tests {
         assert!(paths.len() > 1);
 
         let name = "stratis-test-pool";
-        let (_, mut pool) = StratPool::initialize(name, paths, Redundancy::NONE, None).unwrap();
+        let (_, mut pool) = StratPool::initialize(name, paths, None).unwrap();
         invariant(&pool, name);
 
         assert_eq!(pool.action_avail, ActionAvailability::Full);
@@ -1045,7 +1088,7 @@ mod tests {
         udev_settle().unwrap();
 
         let name = "stratis-test-pool";
-        let (_, mut pool) = StratPool::initialize(name, paths, Redundancy::NONE, None).unwrap();
+        let (_, mut pool) = StratPool::initialize(name, paths, None).unwrap();
         invariant(&pool, name);
 
         assert_eq!(pool.action_avail, ActionAvailability::Full);
